@@ -6,12 +6,14 @@
 
 ###-- Sparse ------------------------------------------------------------
 
-for(f in c("cbind2", "rbind2")) {
-    setMethod(f, signature(x = "sparseMatrix", y = "matrix"),
-	      function(x, y) callGeneric(x, as_Csparse2(y)))
-    setMethod(f, signature(x = "matrix", y = "sparseMatrix"),
-	      function(x, y) callGeneric(as_Csparse2(x), y))
-}
+setMethod("cbind2", signature(x = "sparseMatrix", y = "matrix"),
+	  function(x, y) cbind2(x, .Call(dense_to_Csparse, y)))
+setMethod("cbind2", signature(x = "matrix", y = "sparseMatrix"),
+	  function(x, y) cbind2(.Call(dense_to_Csparse, x), y))
+setMethod("rbind2", signature(x = "sparseMatrix", y = "matrix"),
+	  function(x, y) rbind2(x, .Call(dense_to_Csparse, y)))
+setMethod("rbind2", signature(x = "matrix", y = "sparseMatrix"),
+	  function(x, y) rbind2(.Call(dense_to_Csparse, x), y))
 
 ## originally from ./Matrix.R : -------------------------------
 
@@ -24,9 +26,9 @@ setMethod("cbind2", signature(x = "NULL", y="Matrix"),
           function(x, y) x)
 ## using "atomicVector" not just "numeric"
 setMethod("cbind2", signature(x = "Matrix", y = "atomicVector"),
-	  function(x, y) callGeneric(x, matrix(y, nrow = nrow(x))))
+	  function(x, y) cbind2(x, matrix(y, nrow = nrow(x))))
 setMethod("cbind2", signature(x = "atomicVector", y = "Matrix"),
-	  function(x, y) callGeneric(matrix(x, nrow = nrow(y)), y))
+	  function(x, y) cbind2(matrix(x, nrow = nrow(y)), y))
 setMethod("cbind2", signature(x = "ANY", y = "Matrix"),
 	  function(x, y) .bail.out.2(.Generic, class(x), class(y)))
 setMethod("cbind2", signature(x = "Matrix", y = "ANY"),
@@ -39,9 +41,9 @@ setMethod("rbind2", signature(x = "Matrix", y = "missing"),
 setMethod("rbind2", signature(x = "NULL", y="Matrix"),
           function(x, y) x)
 setMethod("rbind2", signature(x = "Matrix", y = "atomicVector"),
-	  function(x, y) callGeneric(x, matrix(y, ncol = ncol(x))))
+	  function(x, y) rbind2(x, matrix(y, ncol = ncol(x))))
 setMethod("rbind2", signature(x = "atomicVector", y = "Matrix"),
-	  function(x, y) callGeneric(matrix(x, ncol = ncol(y)), y))
+	  function(x, y) rbind2(matrix(x, ncol = ncol(y)), y))
 setMethod("rbind2", signature(x = "ANY", y = "Matrix"),
 	  function(x, y) .bail.out.2(.Generic, class(x), class(y)))
 setMethod("rbind2", signature(x = "Matrix", y = "ANY"),
@@ -93,9 +95,9 @@ setMethod("cbind2", signature(x = "numeric", y = "denseMatrix"),
 
 
 setMethod("cbind2", signature(x = "denseMatrix", y = "matrix"),
-	  function(x, y) callGeneric(x, as_geSimpl(y)))
+	  function(x, y) cbind2(x, as_geSimpl(y)))
 setMethod("cbind2", signature(x = "matrix", y = "denseMatrix"),
-	  function(x, y) callGeneric(as_geSimpl(x), y))
+	  function(x, y) cbind2(as_geSimpl(x), y))
 
 setMethod("cbind2", signature(x = "denseMatrix", y = "denseMatrix"),
 	  function(x, y) {
@@ -140,9 +142,9 @@ setMethod("rbind2", signature(x = "numeric", y = "denseMatrix"),
 	  })
 
 setMethod("rbind2", signature(x = "denseMatrix", y = "matrix"),
-	  function(x, y) callGeneric(x, as_geSimpl(y)))
+	  function(x, y) rbind2(x, as_geSimpl(y)))
 setMethod("rbind2", signature(x = "matrix", y = "denseMatrix"),
-	  function(x, y) callGeneric(as_geSimpl(x), y))
+	  function(x, y) rbind2(as_geSimpl(x), y))
 
 setMethod("rbind2", signature(x = "denseMatrix", y = "denseMatrix"),
 	  function(x, y) {
@@ -188,24 +190,24 @@ setMethod("rbind2", signature(x = "sparseMatrix", y = "diagonalMatrix"),
 for(cls in names(getClass("diagonalMatrix")@subclasses)) {
 
  setMethod("cbind2", signature(x = cls, y = "matrix"),
-	   function(x,y) cbind2(diag2Sp(x), as_Csparse2(y)))
+	   function(x,y) cbind2(diag2Sp(x), .Call(dense_to_Csparse, y)))
  setMethod("cbind2", signature(x = "matrix", y = cls),
-	   function(x,y) cbind2(as_Csparse2(x), diag2Sp(y)))
+	   function(x,y) cbind2(.Call(dense_to_Csparse, x), diag2Sp(y)))
  setMethod("rbind2", signature(x = cls, y = "matrix"),
-	   function(x,y) rbind2(diag2Sp(x), as_Csparse2(y)))
+	   function(x,y) rbind2(diag2Sp(x), .Call(dense_to_Csparse, y)))
  setMethod("rbind2", signature(x = "matrix", y = cls),
-	   function(x,y) rbind2(as_Csparse2(x), diag2Sp(y)))
+	   function(x,y) rbind2(.Call(dense_to_Csparse, x), diag2Sp(y)))
 
  ## These are already defined for "Matrix"
  ## -- repeated here for method dispatch disambiguation	 {"design-FIXME" ?}
  setMethod("cbind2", signature(x = cls, y = "atomicVector"),
-	   function(x, y) callGeneric(x, matrix(y, nrow = nrow(x))))
+	   function(x, y) cbind2(x, matrix(y, nrow = nrow(x))))
  setMethod("cbind2", signature(x = "atomicVector", y = cls),
-	   function(x, y) callGeneric(matrix(x, nrow = nrow(y)), y))
+	   function(x, y) cbind2(matrix(x, nrow = nrow(y)), y))
  setMethod("rbind2", signature(x = cls, y = "atomicVector"),
-	   function(x, y) callGeneric(x, matrix(y, ncol = ncol(x))))
+	   function(x, y) rbind2(x, matrix(y, ncol = ncol(x))))
  setMethod("rbind2", signature(x = "atomicVector", y = cls),
-	   function(x, y) callGeneric(matrix(x, ncol = ncol(y)), y))
+	   function(x, y) rbind2(matrix(x, ncol = ncol(y)), y))
 }
 
 
