@@ -5,12 +5,10 @@
 ##   -----  in  ../src/Mutils.c
 
 setAs("ddenseMatrix", "dgeMatrix",
-      function(from) {
-	  if (class(from) != "dgeMatrix")
-	      .Call(dup_mMatrix_as_dgeMatrix, from)
-	  else from
-      })
+      function(from) .Call(dup_mMatrix_as_dgeMatrix, from))
 
+setAs("ddenseMatrix", "matrix",
+      function(from) as(as(from, "dgeMatrix"), "matrix"))
 
 ## d(ouble) to l(ogical):
 setAs("dgeMatrix", "lgeMatrix", function(from) d2l_Matrix(from, "dgeMatrix"))
