@@ -160,6 +160,10 @@ setMethod("solve", signature(a = "dgeMatrix", b = "matrix"),
 	  function(a, b, ...) .Call(dgeMatrix_matrix_solve, a, b),
           valueClass = "dgeMatrix")
 
+setMethod("solve", signature(a = "dgeMatrix", b = "sparseMatrix"),
+	  function(a, b, ...) .Call(dgeMatrix_matrix_solve, a,
+				    as(b, "denseMatrix")),
+	  valueClass = "dgeMatrix")
 ## not needed - method for numeric defined for Matrix class
 ## setMethod("solve", signature(a = "dgeMatrix", b = "numeric"),
 ## 	  function(a, b, ...)
@@ -177,9 +181,6 @@ setMethod("determinant", signature(x = "dgeMatrix", logarithm = "logical"),
 	  function(x, logarithm, ...)
 	  .Call(dgeMatrix_determinant, x, logarithm))
 
-
-setMethod("expm", signature(x = "dgeMatrix"),
-	  function(x) .Call(dgeMatrix_exp, x),
-	  valueClass = "dgeMatrix")
+##-> ./expm.R  for expm()
 
 ##-> ./colSums.R  for colSums,... rowMeans
