@@ -20,12 +20,10 @@ extern "C" {
 #ifdef __GNUC__
 # undef alloca
 # define alloca(x) __builtin_alloca((x))
-#else
-/* this is necessary (and sufficient) for Solaris 10: */
-#ifdef __sun
+#elif defined(__sun) || defined(_AIX)
+/* this is necessary (and sufficient) for Solaris 10 and AIX 6: */
 # include <alloca.h>
 #endif
-#endif
 
 #define Alloca(n, t)   (t *) alloca( (size_t) ( (n) * sizeof(t) ) )
 
